@@ -48,7 +48,7 @@ const char *opt_output = NULL;
 int opt_margin = 10;
 int opt_markup = FALSE;
 gboolean opt_rtl = FALSE;
-int opt_rotate = 0;
+double opt_rotate = 0;
 gboolean opt_auto_dir = TRUE;
 const char *opt_text = NULL;
 gboolean opt_waterfall = FALSE;
@@ -56,6 +56,7 @@ int opt_width = -1;
 int opt_indent = 0;
 int opt_runs = 1;
 PangoEllipsizeMode opt_ellipsize = PANGO_ELLIPSIZE_NONE;
+PangoGravity opt_gravity = PANGO_GRAVITY_SOUTH;
 HintMode opt_hinting = HINT_DEFAULT;
 PangoWrapMode opt_wrap = PANGO_WRAP_WORD_CHAR;
 gboolean opt_wrap_set = FALSE;
@@ -280,6 +281,7 @@ do_output (PangoContext     *context,
   if (opt_header)
     {
       char *options_string = get_options_string ();
+      pango_context_set_base_gravity (context, PANGO_GRAVITY_SOUTH);
       layout = make_layout (context, options_string, 10);
       pango_layout_get_extents (layout, NULL, &logical_rect);
 
@@ -303,6 +305,8 @@ do_output (PangoContext     *context,
         g_printerr ("The backend does not support rotated text\n");
     }
 
+  pango_context_set_base_gravity (context, opt_gravity);
+
   set_transform (context, transform_cb, cb_context, cb_data, &matrix);
 
   output_body (context,
@@ -378,6 +382,34 @@ parse_ellipsis (const char *name,
   return ret;
 }
 
+static gboolean
+parse_gravity (const char *name,
+	       const char *arg,
+	       gpointer    data,
+	       GError **error)
+{
+  static GEnumClass *class = NULL;
+  gboolean ret = TRUE;
+  GEnumValue *value;
+
+  if (!class)
+    class = g_type_class_ref (PANGO_TYPE_GRAVITY);
+
+  value = g_enum_get_value_by_nick (class, arg);
+  if (value)
+    opt_gravity = value->value;
+  else
+    {
+        g_set_error(error,
+		    G_OPTION_ERROR, 
+		    G_OPTION_ERROR_BAD_VALUE,
+		    "Argument for --gravity must be one of south/east/north/west/auto");
+	ret = FALSE;
+    }
+
+  return ret;
+}
+
 static gboolean
 parse_hinting (const char *name,
 	       const char *arg,
@@ -551,6 +583,8 @@ parse_options (int argc, char *argv[])
      "Ellipsization mode",				  "start/middle/end"},
     {"font",		0, 0, G_OPTION_ARG_STRING,			&opt_font,
      "Set the font description",			       "description"},
+    {"gravity",		0, 0, G_OPTION_ARG_CALLBACK,			&parse_gravity,
+     "Gravity",					"south/east/north/west/auto"},
     {"header",		0, 0, G_OPTION_ARG_NONE,			&opt_header,
      "Display the options in the output",				NULL},
     {"hinting",		0, 0, G_OPTION_ARG_CALLBACK,			&parse_hinting,
@@ -567,7 +601,7 @@ parse_options (int argc, char *argv[])
      "pangorc file to use (default is ./pangorc)",		      "file"},
     {"rtl",		0, 0, G_OPTION_ARG_NONE,			&opt_rtl,
      "Set base direction to right-to-left", 				NULL},
-    {"rotate",		0, 0, G_OPTION_ARG_INT,				&opt_rotate,
+    {"rotate",		0, 0, G_OPTION_ARG_DOUBLE,			&opt_rotate,
      "Angle at which to rotate results",		 	   "degrees"},
     {"runs",		'n', 0, G_OPTION_ARG_INT,			&opt_runs,
      "Run Pango layout engine this many times",			   "integer"},
@@ -587,7 +621,6 @@ parse_options (int argc, char *argv[])
   GError *parse_error = NULL;
   GOptionContext *context;
   size_t len;
-  char *p;
 
   prog_name = g_path_get_basename (argv[0]);
   context = g_option_context_new ("- FILE");
