@@ -30,8 +30,8 @@
 
 #include "pango-enum-types.h"
 #include "pango-modules.h"
-#include "pango-utils.h"
 #include "pango-impl-utils.h"
+#include "modules.h"
 
 typedef struct _PangoModule      PangoModule;
 typedef struct _PangoModuleClass PangoModuleClass;
@@ -110,7 +110,7 @@ static GType pango_module_get_type (void);
  * type. The resulting map can be used to determine the engine
  * for each character.
  * 
- * Return value: 
+ * Return value: the suitable #PangoMap.
  **/
 PangoMap *
 pango_find_map (PangoLanguage *language,
@@ -138,7 +138,7 @@ pango_find_map (PangoLanguage *language,
 
   if (!tmp_list)
     {
-      map_info = g_new (PangoMapInfo, 1);
+      map_info = g_slice_new (PangoMapInfo);
       map_info->language = language;
       map_info->engine_type_id = engine_type_id;
       map_info->render_type_id = render_type_id;
@@ -248,7 +248,7 @@ pango_module_class_init (PangoModuleClass *class)
 
 static PANGO_DEFINE_TYPE (PangoModule, pango_module,
 			  pango_module_class_init, NULL,
-			  G_TYPE_TYPE_MODULE);
+			  G_TYPE_TYPE_MODULE)
 
 static PangoEngine *
 pango_engine_pair_get_engine (PangoEnginePair *pair)
@@ -262,7 +262,25 @@ pango_engine_pair_get_engine (PangoEnginePair *pair)
 	}
 
       if (!pair->engine)
-	g_printerr ("Failed to load Pango module for id: '%s'", pair->info.id);
+        {
+	  /* If a module cannot be used, or doesn't not create an engine
+	   * correctly, we print out an error containing module name and id,
+	   * but to not flood the terminal with zillions of the message, we
+	   * set a flag on the module to only err once per module.
+	   */
+	  static GQuark warned_quark = 0;
+
+	  if (!warned_quark)
+	    warned_quark = g_quark_from_static_string ("pango-module-warned");
+	  
+	  if (!g_object_get_qdata (G_OBJECT (pair->module), warned_quark))
+	    {
+	      g_warning ("Failed to load Pango module '%s' for id '%s'", pair->module->path, pair->info.id);
+
+	      g_object_set_qdata_full (G_OBJECT (pair->module), warned_quark,
+				       GINT_TO_POINTER (1), NULL);
+	    }
+	}
     }
 
   return pair->engine;
@@ -286,7 +304,7 @@ handle_included_module (PangoIncludedModule *included_module,
 
   for (i = 0; i < n_engines; i++)
     {
-      PangoEnginePair *pair = g_new (PangoEnginePair, 1);
+      PangoEnginePair *pair = g_slice_new (PangoEnginePair);
 
       pair->info = engine_info[i];
       pair->module = module;
@@ -354,6 +372,12 @@ script_from_string (const char *str)
   return value->value;
 }
 
+static void
+script_info_free (PangoEngineScriptInfo *script_info, gpointer data)
+{
+  g_slice_free (PangoEngineScriptInfo, script_info);
+}
+
 static gboolean /* Returns true if succeeded, false if failed */
 process_module_file (FILE *module_file)
 {
@@ -363,7 +387,7 @@ process_module_file (FILE *module_file)
 
   while (pango_read_line (module_file, line_buf))
     {
-      PangoEnginePair *pair = g_new (PangoEnginePair, 1);
+      PangoEnginePair *pair = g_slice_new (PangoEnginePair);
       PangoEngineScriptInfo *script_info;
       PangoScript script;
       GList *scripts = NULL;
@@ -377,7 +401,7 @@ process_module_file (FILE *module_file)
 
       if (!pango_skip_space (&p))
 	{
-	  g_free (pair);
+	  g_slice_free (PangoEnginePair, pair);
 	  continue;
 	}
 
@@ -419,7 +443,7 @@ process_module_file (FILE *module_file)
 		  goto error;
 		}
 	      
-	      script_info = g_new (PangoEngineScriptInfo, 1);
+	      script_info = g_slice_new (PangoEngineScriptInfo);
 	      script_info->script = script;
 	      script_info->langs = g_strdup (q + 1);
 	      
@@ -454,13 +478,13 @@ process_module_file (FILE *module_file)
       dlloaded_engines = g_slist_prepend (dlloaded_engines, pair);
 
     error:
-      g_list_foreach (scripts, (GFunc)g_free, NULL);
+      g_list_foreach (scripts, (GFunc)script_info_free, NULL);
       g_list_free (scripts);
 
       if (have_error)
 	{
 	  g_printerr ("Error reading Pango modules file\n");
-	  g_free(pair);
+	  g_slice_free(PangoEnginePair, pair);
 	  break;
 	}
     }
@@ -513,6 +537,7 @@ static void
 init_modules (void)
 {
   static gboolean init = FALSE;
+  int i;
 
   if (init)
     return;
@@ -522,6 +547,8 @@ init_modules (void)
   /* Make sure that the type system is initialized */
   g_type_init ();
   
+  for (i = 0; _pango_included_lang_modules[i].list; i++)
+    pango_module_register (&_pango_included_lang_modules[i]);
   read_modules ();
 }
 
@@ -581,8 +608,6 @@ map_add_engine_list (PangoMapInfo *info,
 static void
 build_map (PangoMapInfo *info)
 {
-  PangoMap *map;
-
   const char *engine_type = g_quark_to_string (info->engine_type_id);
   const char *render_type = g_quark_to_string (info->render_type_id);
   
@@ -596,11 +621,11 @@ build_map (PangoMapInfo *info)
 	  gchar *filename = g_build_filename (pango_get_sysconf_subdirectory (),
 					      "pango.modules",
 					      NULL);
-	  g_warning ("No builtin or dynamically loaded modules\n"
-		     "were found. Pango will not work correctly. This probably means\n"
-		     "there was an error in the creation of:\n"
+	  g_warning ("No builtin or dynamically\n"
+		     "loaded modules were found. Pango will not work correctly.\n"
+		     "This probably means there was an error in the creation of:\n"
 		     "  '%s'\n"
-		     "You may be able to recreate this file by running pango-querymodules.",
+		     "You should create this file by running pango-querymodules.",
 		     filename);
 	  g_free (filename);
 	  
@@ -608,8 +633,8 @@ build_map (PangoMapInfo *info)
 	}
     }
   
-  info->map = map = g_new (PangoMap, 1);
-  map->entries = g_array_new (FALSE, TRUE, sizeof (PangoMapEntry));
+  info->map = g_slice_new (PangoMap);
+  info->map->entries = g_array_new (FALSE, TRUE, sizeof (PangoMapEntry));
 			      
   map_add_engine_list (info, dlloaded_engines, engine_type, render_type);  
   map_add_engine_list (info, registered_engines, engine_type, render_type);  
