@@ -46,7 +46,7 @@ typedef struct _GlyphExtentsCacheEntry    GlyphExtentsCacheEntry;
 #define GLYPH_CACHE_NUM_ENTRIES 256 /* should be power of two */
 #define GLYPH_CACHE_MASK (GLYPH_CACHE_NUM_ENTRIES - 1)
 
-#define PANGO_UNITS(Double) ((int)((Double) * PANGO_SCALE + 0.49999))
+#define PANGO_UNITS(Double) ((int)floor((Double) * PANGO_SCALE + 0.5))
 
 /* An entry in the fixed-size cache for the glyph -> ink_rect mapping.
  * The cache is indexed by the lower N bits of the glyph (see
@@ -69,6 +69,7 @@ struct _PangoCairoFcFont
   cairo_matrix_t font_matrix;
   cairo_matrix_t ctm;
   cairo_font_options_t *options;
+  PangoGravity gravity;
 
   PangoRectangle font_extents;
   GlyphExtentsCacheEntry    *glyph_extents_cache;  
@@ -133,7 +134,7 @@ pango_cairo_fc_font_get_scaled_font (PangoCairoFont *font)
       if (!cffont->scaled_font)
         return NULL;
     }
-  
+
   return cffont->scaled_font;
 }
 
@@ -215,6 +216,7 @@ pango_cairo_fc_font_get_metrics (PangoFont     *font,
   if (!tmp_list)
     {
       PangoContext *context;
+      int height, shift;
 
       if (!fcfont->fontmap)
 	return pango_font_metrics_new ();
@@ -230,6 +232,30 @@ pango_cairo_fc_font_get_metrics (PangoFont     *font,
       pango_cairo_context_set_font_options (context, cffont->options);
 
       info->metrics = pango_fc_font_create_metrics_for_context (fcfont, context);
+      /* We may actually reuse ascent/descent we got from cairo here.  that's
+       * in cffont->font_extents.
+       */
+      height = info->metrics->ascent + info->metrics->descent;
+      switch (cffont->gravity)
+	{
+	  default:
+	  case PANGO_GRAVITY_AUTO:
+	  case PANGO_GRAVITY_SOUTH:
+	    break;
+	  case PANGO_GRAVITY_NORTH:
+	    info->metrics->ascent = info->metrics->descent;
+	    break;
+	  case PANGO_GRAVITY_EAST:
+	  case PANGO_GRAVITY_WEST:
+	    info->metrics->ascent = height / 2;
+	}
+      shift = (height - info->metrics->ascent) - info->metrics->descent;
+      if (fcfont->is_hinted)
+        shift &= ~(PANGO_SCALE - 1);
+      info->metrics->descent += shift;
+      info->metrics->underline_position -= shift;
+      info->metrics->strikethrough_position -= shift;
+      info->metrics->ascent = height - info->metrics->descent;
 
       g_object_unref (context);
     }
@@ -269,9 +295,22 @@ pango_cairo_fc_font_glyph_extents_cache_init (PangoCairoFcFont *cffont)
   cairo_scaled_font_extents (scaled_font, &font_extents);
 
   cffont->font_extents.x = 0;
-  cffont->font_extents.y = - PANGO_UNITS (font_extents.ascent);
-  cffont->font_extents.height = PANGO_UNITS (font_extents.ascent + font_extents.descent);
   cffont->font_extents.width = 0;
+  cffont->font_extents.height = PANGO_UNITS (font_extents.ascent + font_extents.descent);
+  switch (cffont->gravity)
+    {
+      default:
+      case PANGO_GRAVITY_AUTO:
+      case PANGO_GRAVITY_SOUTH:
+        cffont->font_extents.y = - PANGO_UNITS (font_extents.ascent);
+	break;
+      case PANGO_GRAVITY_NORTH:
+        cffont->font_extents.y = - PANGO_UNITS (font_extents.descent);
+	break;
+      case PANGO_GRAVITY_EAST:
+      case PANGO_GRAVITY_WEST:
+        cffont->font_extents.y = - PANGO_UNITS ((font_extents.ascent + font_extents.descent) * 0.5);
+    }
 
   cffont->glyph_extents_cache = g_new0 (GlyphExtentsCacheEntry, GLYPH_CACHE_NUM_ENTRIES);
   /* Make sure all cache entries are invalid initially */
@@ -468,6 +507,8 @@ _pango_cairo_fc_font_new (PangoCairoFcFontMap        *cffontmap,
 			 "pattern", pattern,
 			 NULL);
 
+  cffont->gravity = pango_font_description_get_gravity (desc);
+
   if  (FcPatternGetMatrix (pattern,
 			   FC_MATRIX, 0, &fc_matrix) == FcResultMatch)
     {
@@ -481,6 +522,9 @@ _pango_cairo_fc_font_new (PangoCairoFcFontMap        *cffontmap,
   else
     cairo_matrix_init_identity (&cffont->font_matrix);
 
+  cairo_matrix_rotate(&cffont->font_matrix,
+		      pango_gravity_to_rotation (cffont->gravity));
+
   pango_ctm = pango_context_get_matrix (context);
 
   size = get_font_size (cffontmap, context, desc, pattern, pango_ctm);
