@@ -36,7 +36,7 @@ pango_cairo_font_get_type (void)
 
   if (! cairo_font_type)
     {
-      static const GTypeInfo cairo_font_info =
+      const GTypeInfo cairo_font_info =
       {
 	sizeof (PangoCairoFontIface), /* class_size */
 	NULL,           /* base_init */
@@ -90,7 +90,6 @@ cairo_font_face_t *
 _pango_cairo_font_get_font_face (PangoCairoFont *font)
 {
   g_return_val_if_fail (PANGO_IS_CAIRO_FONT (font), NULL);
-  /* this function will be removed.  don't bother with warning history here */
   
   return (* PANGO_CAIRO_FONT_GET_IFACE (font)->get_font_face) (font);
 }
@@ -98,15 +97,7 @@ _pango_cairo_font_get_font_face (PangoCairoFont *font)
 cairo_scaled_font_t *
 _pango_cairo_font_get_scaled_font (PangoCairoFont *font)
 {
-  if (G_UNLIKELY (!PANGO_IS_CAIRO_FONT (font)))
-    {
-      if (!_pango_cairo_warning_history.font_get_scaled_font)
-        {
-	  _pango_cairo_warning_history.font_get_scaled_font = TRUE;
-	  g_warning ("_pango_cairo_font_get_scaled_font called with bad font, expect ugly output");
-	}
-      return NULL;
-    }
+  g_return_val_if_fail (PANGO_IS_CAIRO_FONT (font), NULL);
   
   return (* PANGO_CAIRO_FONT_GET_IFACE (font)->get_scaled_font) (font);
 }
@@ -131,7 +122,7 @@ _pango_cairo_font_get_hex_box_info (PangoCairoFont *cfont)
   PangoCairoHexBoxInfo *hbi;
 
   /* for metrics hinting */
-  double scale_x = 1., scale_x_inv = 1., scale_y = 1., scale_y_inv = 1.;
+  double scale_x, scale_x_inv, scale_y, scale_y_inv;
   gboolean is_hinted;
 
   int i;
@@ -226,7 +217,7 @@ _pango_cairo_font_get_hex_box_info (PangoCairoFont *cfont)
 
     /* load mini_font */
 
-    context = pango_cairo_font_map_create_context ((PangoCairoFontMap *) (fontmap));
+    context = pango_cairo_font_map_create_context (PANGO_CAIRO_FONT_MAP (fontmap));
 
     pango_context_set_matrix (context, &pango_ctm);
     pango_context_set_language (context, pango_language_from_string ("en"));
